/*
 *   Copyright            : (C) 2003 by Jakob Schroeter
 *   License              : This file is released under the terms of the LGPL, version 2.
 *   Email                : js@camaya.de
 */


#include <qpainter.h>
#include <qfont.h>
#include <qapplication.h>
#include <qbitmap.h>
#include <qimage.h>
#include <qcolor.h>
#include <qfont.h>
#include <qtimer.h>
#include <qsize.h>
#include <qsimplerichtext.h>
#include <qpalette.h>

#include <kconfig.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <kdialog.h>

#include "konscreendisplay.h"
#include "konscreendisplay.moc"

KOnScreenDisplay::KOnScreenDisplay( QWidget *parent, const char *name )
          : QWidget( parent, name, WStyle_NoBorder | WX11BypassWM | WStyle_Customize |
                                   WRepaintNoErase | WStyle_Tool | WStyle_StaysOnTop )
{
    m_parent = parent;
    m_parentWidth = ( m_parent == 0L )?( KGlobalSettings::splashScreenDesktopGeometry().width() ):( m_parent->width() );
    m_parentHeight = ( m_parent == 0L )?( KGlobalSettings::splashScreenDesktopGeometry().height() ):( m_parent->height() );
    m_timeout = 0;
    m_text = QString::null;
    m_position = Qt::AlignHCenter | Qt::AlignVCenter;
    m_iconPosition = Qt::AlignLeft;
    m_horizontalOff = 0;
    m_verticalOff = 0;
    m_fgColor = QColor();
    m_bgColor = QColor();
    m_shadow = true;
    m_shadowOffset = 1;
    m_scroll = false;
    m_autoDelete = false;
    m_blinkFreq = 0;
    m_blinkTime = 0;
    m_widget = 0;
    m_layout = 0;
    m_jump = false;
    m_moveDir = KOnScreenDisplay::ToTop;

    setFocusPolicy( QWidget::NoFocus );
}

KOnScreenDisplay::~KOnScreenDisplay()
{
    m_widget = 0;
    m_layout = 0;
}

KOnScreenDisplay *KOnScreenDisplay::message( QWidget *parent, const QString &text )
{
    KOnScreenDisplay *kosd = new KOnScreenDisplay( parent );
    kosd->loadConfig();
    kosd->setText( text );
    kosd->show();

    return kosd;
}

KOnScreenDisplay *KOnScreenDisplay::message( QWidget *parent, const QString &text, const QPixmap &icon, const unsigned int color )
{
    KOnScreenDisplay *kosd = new KOnScreenDisplay( parent );
    kosd->loadConfig();

    if (color)
      kosd->setFgColor( QColor( color ) );

    kosd->setText( text );
    kosd->setIcon( icon );
    kosd->show();

    return kosd;
}

void KOnScreenDisplay::create()
{
    QSimpleRichText rt( m_text, font() );
    QColorGroup cg;
    cg.setColor( QColorGroup::Base, Qt::color0 );
    QRect rc_i;
    int textX = 0, x, w = KGlobalSettings::splashScreenDesktopGeometry().width();
    int textY = 0, y, h = KGlobalSettings::splashScreenDesktopGeometry().height();

    if ( !m_icon.isNull() )
       rc_i.setRect( 0, 0, m_icon.width(), m_icon.height() );

    w = rt.width();
    h = rt.height();

    if ( m_shadow )
    {
      w += m_shadowOffset;
      h += m_shadowOffset;
    }

    if ( !m_icon.isNull() )
    {

      if ( m_iconPosition & Qt::AlignRight )
        rc_i.moveLeft( rc_i.left() + w + m_iconSpacer );

      else if ( m_iconPosition & Qt::AlignLeft )
        textX = m_icon.width() + m_iconSpacer;

      w += m_icon.width() + m_iconSpacer;
      if ( h < m_icon.height() )
      {
        h = m_icon.height();
        textY = (h / 2) - (rt.height() / 2);
      }
      else
        rc_i.moveTop( rc_i.top() + (h / 2) - (m_icon.height() / 2) );
    }
    resize( QSize(w, h) );

    positionSelf();

    w = width(); h = height(); x = this->x(); y = this->y();
    qApp->syncX();
    QPixmap bgpic( w, h );
    QPainter p;
    if ( m_transparency )
    {
      QBitmap mask( w, h );
      p.begin( &mask );
      p.fillRect( 0, 0, w, h, Qt::color0 );
      cg.setColor( QColorGroup::Text, Qt::color1 );
      rt.draw( &p, textX, textY, QRect(), cg );
      if ( m_shadow )
        rt.draw( &p, textX + m_shadowOffset, textY + m_shadowOffset, QRect(), cg );

      if ( !m_icon.isNull() )
        p.drawPixmap( rc_i, m_icon.convertToImage().createAlphaMask() );

      p.end();
      setMask( mask );
    }
    else
    {
      p.begin( &bgpic );
      p.fillRect( 0, 0, w, h, m_bgColor );
      p.end();
    }

    p.begin( &bgpic );
    p.setFont( font() );
    if ( !m_icon.isNull() )
      p.drawPixmap( rc_i, m_icon );

    if ( m_shadow )
    {
      cg.setColor( QColorGroup::Text, QColor( 0,0,0 ) );
      rt.draw( &p, textX + m_shadowOffset, textY + m_shadowOffset, QRect(), cg );
    }

    cg.setColor( QColorGroup::Text, m_fgColor );
    rt.draw( &p, textX, textY, QRect(), cg );
    p.end();
    m_bgpic = bgpic;
}

void KOnScreenDisplay::positionSelf()
{
    int x, y;

    if ( m_position & Qt::AlignLeft )
    {
      x = m_horizontalOff;
    }
    else if ( m_position & Qt::AlignHCenter )
    {
      x = (m_parentWidth / 2) - (width() / 2) + m_horizontalOff;
    }
    else if ( m_position & Qt::AlignRight )
    {
      x = m_parentWidth - width() - m_horizontalOff;
    }
    else
    {
      x = 0;
    }

    if ( m_position & Qt::AlignTop )
    {
      y = m_verticalOff;
    }
    else if ( m_position & Qt::AlignVCenter )
    {
      y = (m_parentHeight / 2) - (height() / 2) + m_verticalOff;
    }
    else if ( m_position & Qt::AlignBottom )
    {
      y = m_parentHeight - height() - m_verticalOff;
    }
    else
    {
      y = 0;
    }

    if ( x < 0 ) x = 0;
    if ( y < 0 ) y = 0;

    move( x, y );
}

void KOnScreenDisplay::setPosition( const int position, const int verticalOff, const int horizontalOff )
{
    m_position = position;
    m_horizontalOff = horizontalOff;
    m_verticalOff = verticalOff;

    m_configChanged = true;
}

void KOnScreenDisplay::setTimeout( const int timeout )
{
    m_timeout = timeout;

    m_configChanged = true;
}

void KOnScreenDisplay::setFgColor( const QColor &fgColor )
{
    m_fgColor = fgColor;
}

void KOnScreenDisplay::setBgColor(const QColor &bgColor)
{
    m_bgColor = bgColor;

    m_configChanged = true;
}

void KOnScreenDisplay::setText(const QString &text)
{
    //m_text = QString::fromLatin1("<qt><nobr>") + text + QString::fromLatin1("</nobr></qt>");
    m_text = text;

    m_configChanged = true;
}

void KOnScreenDisplay::setShadow(const bool shadow, const int offset)
{
    m_shadow = (offset)?(shadow):(false);
    m_shadowOffset = offset;

    m_configChanged = true;
}

void KOnScreenDisplay::setTransparency(const bool transparency)
{
    m_transparency = transparency;
}

void KOnScreenDisplay::setScrolling(const bool scroll, const int scrollSpeed)
{
    m_scroll = scroll;
    m_scrollSpeed = scrollSpeed;

    if ( m_scroll )
    {
      scrollTimer = new QTimer( this );
      connect( scrollTimer, SIGNAL(timeout()), SLOT(slotScroll()) );
      scrollTimer->start( m_scrollSpeed , false );
    }

    m_configChanged = true;
}

void KOnScreenDisplay::setMoving(const int moveDir)
{
    m_moveDir = moveDir;

    m_configChanged = true;
}

void KOnScreenDisplay::setBlinking(const int time, const int freq)
{
    m_blinkFreq = freq;
    m_blinkTime = time;

    if (m_blinkTime && m_blinkFreq)
    {
      blinkTimer = new QTimer( this );
      connect( blinkTimer, SIGNAL(timeout()), SLOT(slotBlink()) );
      blinkTimer->start( m_blinkFreq, true );

      QTimer::singleShot( m_blinkTime, this, SLOT(slotBlinkTimeout()) );
    }

    m_configChanged = true;
}

void KOnScreenDisplay::setJumping(const bool jump)
{
    m_jump = jump;

    m_configChanged = true;
}

void KOnScreenDisplay::setIcon(const QPixmap icon, const int position, const int spacer)
{
    m_icon = icon;
    m_iconPosition = position;
    m_iconSpacer = spacer;

    m_configChanged = true;
}

void KOnScreenDisplay::setAutoDelete(const bool autoDelete)
{
  m_autoDelete = autoDelete;

  m_configChanged = true;
}

void KOnScreenDisplay::show()
{
    create();
    QWidget::show();
    QWidget::raise();
    
    if ( m_timeout )
      QTimer::singleShot( m_timeout * 1000, this, SLOT(slotTimeout()) );
}

void KOnScreenDisplay::paintEvent( QPaintEvent* )
{
    repaint();
}

void KOnScreenDisplay::slotTimeout()
{
    if ( m_scroll )
      scrollTimer->stop();

    if (m_blinkTime)
      blinkTimer->stop();

    hide();
    emit signalEnd();

    if ( m_autoDelete )
      deleteLater();
}

void KOnScreenDisplay::slotScroll()
{
    if ( m_blinkTime )
    {
      int y = getNewYPosition( this->y(), 1 );

      if ( y < (0-height()) )
        slotTimeout();
      else
        move( this->x(), y );
    }
    else
      scrollTimer->stop();
}

void KOnScreenDisplay::slotBlink()
{
    blinkTimer->stop();
    setHidden( !isHidden() );
    blinkTimer->start( m_blinkFreq, true );
}

void KOnScreenDisplay::slotBlinkTimeout()
{
    blinkTimer->stop();
    setShown( true );
}

void KOnScreenDisplay::slotJump()
{
    if ( m_jump )
      move( this->x(), getNewYPosition( this->y(), height() + 5 ) );
}

void KOnScreenDisplay::slotRefreshOSD()
{
    repaint();
}

int KOnScreenDisplay::getNewYPosition( int oldPos, const int step )
{
    switch ( m_moveDir ) {
      case ToBottom:
        oldPos += step;
        break;
      case ToTop:
        oldPos -= step;
        break;
    }

    return oldPos;
}

void KOnScreenDisplay::repaint()
{
    m_parentWidth = ( m_parent == 0L )?( KGlobalSettings::splashScreenDesktopGeometry().width() ):( m_parent->width() );
    m_parentHeight = ( m_parent == 0L )?( KGlobalSettings::splashScreenDesktopGeometry().height() ):( m_parent->height() );

    if ( m_configChanged )
      create();
    else
      positionSelf();

    m_configChanged = false;

    QPainter p( this );
    p.drawPixmap( 0, 0, m_bgpic );
    p.end();
}

void KOnScreenDisplay::loadConfig()
{
    KConfig b( ".kderc", false, true );
    b.setGroup( "General" );
    setFont( b.readFontEntry( "onScreenDisplayFont", 0L ) );

    KConfig c( "kosdrc", false, false );
    setTimeout( c.readUnsignedNumEntry( "Timeout", 5 ) );
    setPosition( c.readUnsignedNumEntry( "Position", 4 ),
                 c.readUnsignedNumEntry( "VerticalOffset", 0 ),
                 c.readUnsignedNumEntry( "HorizontalOffset", 0 ) );
    setShadow( c.readBoolEntry( "TextShadow", true ),
               c.readUnsignedNumEntry( "ShadowSize", 1 ) );
    setFgColor( c.readColorEntry( "ForegroundColor", 0L ) );
    setBgColor( c.readColorEntry( "BackgroundColor", 0L ) );
    setTransparency( c.readBoolEntry( "TransparentBackground", true ) );
    setMoving( c.readUnsignedNumEntry( "MoveDirection", 0 ) );
    setScrolling( c.readBoolEntry( "Scrolling", false ),
                  c.readUnsignedNumEntry( "ScrollSpeed", 50 ) );
    setBlinking( c.readUnsignedNumEntry( "BlinkTime", 0 ),
                 c.readUnsignedNumEntry( "BlinkFreq", 0 ) );
    setJumping( c.readBoolEntry( "Jumping", true ) );
}
